//  ListAdditions.h
//
//  IMPORTANT: This code was developed by Wade Tregaskis under the employment of the Department of Primary Industries, Victoria, whom reserve all rights to it.  It has been used in good faith by Wade Tregaskis without explicit permission, but with the understanding from the employer that it's private use is acceptable in good faith.
//
//  The underlying message being, don't give this to anyone please as I don't have the rights for it's distribution.  Also, although I did write every character of it, I did so outside of university and as such I understand it may not be eligible for inclusion in university marking and other evaluation.

#ifndef LIST_ADDITIONS_H
#define LIST_ADDITIONS_H

#include <list>
using namespace std;


/*! @header List Additions
    @abstract Provides a few additional functions for working with STL lists.
    @discussion Stupidly, the STL list doesn't include functions for checking if a list contains an object by reference to the object, nor for removing from the list by reference to the object.  Thus, equivalent methods are implemented here.
    */
    
/*! @function listContains
    @abstract Determines whether a given list contains a given object.
    @param l The list.
    @param object The object.
    @result Returns true if the given list contains the given object, false otherwise. */

template<class T> bool listContains(const list<T> *l, T object) {
	typename list<T>::const_iterator iter, stopper;

	for (iter = l->begin(), stopper = l->end(); iter != stopper; ++iter) {
		if (*iter == object) {
			return true;
		}
	}

	return false;
}

/*! @function remove
    @abstract Removes an object from a given list.
    @discussion Searches through the list 'l' for all occurances of 'object', removing them as it goes.
    @param l The list search through.
    @param object The object to remove. */

template<class T> void remove(list<T> *l, T object) {
	typename list<T>::iterator iter = l->begin();

	while (iter != l->end()) {
		if (*iter == object) {
			iter = l->erase(iter);
		} else {
			++iter;
		}
	}
}

#endif
