//  RRScheduler.cpp
//  Scheduler Simulator
//
//  Created by Wade Tregaskis on Fri Apr 09 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "RRScheduler.h"


void RRScheduler::runNextProcess(void) {
    if (active) {
        ProcessInfo *current;
        
        if (!toRun.empty()) {
            current = toRun.front();
            toRun.pop_front();
            
            executing.push_back(current);
            
            if (0 != gettimeofday(&(current->firstServed), NULL)) {
                fprintf(stderr, "RRScheduler::runNextProcess() - Unable to get time of day, error #%d.\n", errno);
            }
            
            current->pid = spawnChild(current->executionTime);
            
            if (current->pid <= 0) {
                fprintf(stderr, "RRScheduler::runNextProcess() - Unable to spawn process, error #%d.\n", errno);
            }
        } else if (!ready.empty()) {
            current = ready.front();
            ready.pop_front();
            
            executing.push_back(current);
            
            if (0 != kill(current->pid, SIGCONT)) {
                fprintf(stderr, "RRScheduler::runNextProcess() - Unable to wake up child process #%d - error #%d.\n", current->pid, errno);
            }
        }
    }
}

RRScheduler::RRScheduler(bool inter) : FiveQueueScheduler(inter) {
    // Do nothing
}

RRScheduler::~RRScheduler(void) {
    // Do nothing
}

void RRScheduler::tick(void) {
    list<ProcessInfo*>::iterator iter, stopper;
    int count = 0;
    ProcessInfo *current;
    
    ++ticksSoFar;
    
    ready.insert(ready.end(), blocked.begin(), blocked.end());
    blocked.clear();
    
    for (iter = executing.begin(), stopper = executing.end(); iter != stopper; ++iter) {
        current = *iter;
        
        current->executionTimeRemaining -= currentTimeQuantum;
        
        if (0 != kill(current->pid, SIGSTOP)) {
            fprintf(stderr, "RRScheduler::tick() - Unable to stop process #%d, error #%d.\n", current->pid, errno);
        } else {
            ++count;
        }
        
        assert(current);
        ready.push_back(current);
    }
    
    executing.clear();
    
    while (count > 0) {
        this->runNextProcess();
        --count;
    }
    
    if (interactive) {
        this->updateInteractiveDisplay();
    }
}
