//  RRScheduler.h
//  Scheduler Simulator
//
//  Created by Wade Tregaskis on Fri Apr 09 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef RR_SCHEDULER_H
#define RR_SCHEDULER_H

#include "FiveQueueScheduler.h"


/*! @class RRScheduler
    @abstract RR (Round Robin) Scheduler.
    @discussion The Round Robin scheduler allocates one time quantum of processor time to each task in the order they are first submitted to it, repeating the predictable allocation over and over again.  It ignores priority both internally and at it's inputs.  This is the 'fair' implementation of an alternative (better) version, PRR (see the PRRScheduler class).
    */

class RRScheduler : public FiveQueueScheduler {
protected:
    // FiveQueueScheduler
    
    virtual void runNextProcess(void);

public:
    RRScheduler(bool inter = false);
    ~RRScheduler(void);
    
    // Scheduler
    
    virtual void tick(void);
};

#endif
