//  SJFScheduler.h
//  Scheduler Simulator
//
//  Created by Wade Tregaskis on Fri Apr 09 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef SJF_SCHEDULER_H
#define SJF_SCHEDULER_H

#include "FiveQueueScheduler.h"
#include "ListAdditions.h"


/*! @class SJFScheduler
    @abstract SJF (Shortest Job First) Scheduler.
    @discussion The SJF looks at all available to-run jobs and chooses the [supposedly] shortest one.  It then executes that job to completion.  It then repeats the whole process.  If a process blocks it is placed into the ready queue and the next shortest begins execution.  The first process will be able to resume once the second process finishes (or blocks itself).  This is a 'CPU-optimum' SJF scheduler, not a 'strict' one.
    */

class SJFScheduler : public FiveQueueScheduler {
protected:
    // FiveQueueScheduler
    
    virtual void runNextProcess(void);

public:
    SJFScheduler(bool inter = false);
    ~SJFScheduler(void);
};

#endif
