//  TimeUtils.c
//  Scheduler Simulator
//
//  Created by Wade Tregaskis on Sat Apr 10 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "TimeUtils.h"


double timeDifference(struct timeval first, struct timeval second) {
    return (first.tv_sec + (first.tv_usec / 1000000.0)) - (second.tv_sec + (second.tv_usec / 1000000.0));
}

#define BUFFER_SIZE 100
static char timeBuffer[BUFFER_SIZE], buffer[BUFFER_SIZE];

const char* timeAsString(struct timeval time) {    
    time_t timeVal;

    timeVal = (int)round(time.tv_sec + (time.tv_usec / 1000000.0));

    ctime_r(&timeVal, timeBuffer);
    
    for (int i = 0; i < BUFFER_SIZE; ++i) {
        if (timeBuffer[i] == '\0') {
            return timeBuffer;
        } else if (timeBuffer[i] == '\n') {
            timeBuffer[i] = '\0';
            return timeBuffer;
        }
    }
    
    return NULL;
}

const char* timeSinceNowAsString(struct timeval time) {
    struct timeval currentTime;
    double difference;
    const char *beforeAfter;
    
    if (0 != gettimeofday(&currentTime, NULL)) {
        buffer[0] = '\0';
    } else {
        difference = (currentTime.tv_sec + (currentTime.tv_usec / 1000000.0)) - (time.tv_sec + (time.tv_usec / 1000000.0));
        
        if (difference < 0.0) { // Ahead of present time
            difference = fabs(difference);
            beforeAfter = "from now";
        } else { // Prior to present time
            beforeAfter = "ago";
        }
        
        int days, hours, minutes, seconds, milliseconds, microseconds;
        
        days = (int)floor(difference / 86400.0);
        difference -= days * 86400.0;
        
        hours = (int)floor(difference / 3600.0);
        difference -= hours * 3600.0;
        
        minutes = (int)floor(difference / 60.0);
        difference -= minutes * 60.0;
        
        seconds = (int)floor(difference);
        difference -= seconds;
        
        milliseconds = (int)floor(difference * 1000.0);
        difference -= milliseconds / 1000.0;
        
        microseconds = (int)floor(difference * 1000000.0);
        
        if (milliseconds > 0) { // At least milliseconds
            if (seconds > 0) { // At least seconds
                if (minutes > 0) { // At least minutes
                    if (hours > 0) { // At least hours
                        if (days > 0) { // At least days
                            snprintf(buffer, BUFFER_SIZE, "%d days, %d hours, %d minutes, %d seconds, %d milliseconds & %d microseconds %s", days, hours, minutes, seconds, milliseconds, microseconds, beforeAfter);
                        } else { // Only hours
                            snprintf(buffer, BUFFER_SIZE, "%d hours, %d minutes, %d seconds, %d milliseconds & %d microseconds %s", hours, minutes, seconds, milliseconds, microseconds, beforeAfter);
                        }
                    } else { // Only minutes
                        snprintf(buffer, BUFFER_SIZE, "%d minutes, %d seconds, %d milliseconds & %d microseconds %s", minutes, seconds, milliseconds, microseconds, beforeAfter);
                    }
                } else { // Only seconds
                    snprintf(buffer, BUFFER_SIZE, "%d seconds, %d milliseconds & %d microseconds %s", seconds, milliseconds, microseconds, beforeAfter);
                }
            } else { // Only milliseconds
                snprintf(buffer, BUFFER_SIZE, "%d milliseconds & %d microseconds %s", milliseconds, microseconds, beforeAfter);
            }
        } else { // Only microseconds
            snprintf(buffer, BUFFER_SIZE, "%d microseconds %s", microseconds, beforeAfter);
        }
    }
    
    return buffer;
}
