//  TimeUtils.h
//  Scheduler Simulator
//
//  Created by Wade Tregaskis on Sat Apr 10 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef TIME_UTILS_H
#define TIME_UTILS_H

#include <sys/time.h>
#include <sys/resource.h>
#include <time.h>

#include <math.h>

#include <stdio.h>

/*! @function timeDifference
    @abstract Returns the number of seconds between two times.
    @param first The first time.
    @param second The second time.
    @result Returns the number of seconds between the two, which is positive if the second is earlier than the first, negative otherwise (think "first - second = ..."). */

double timeDifference(struct timeval first, struct timeval second);

/*! @function timeAsString
    @abstract Returns a user-friendly string representation of the given time & date.
    @discussion Warning: the returned string points into an internal buffer.  Subsequent calls to this function will use the same buffer, overwriting existing values.  Make sure to copy or otherwise use this function after calling it, before using it again.
    @param time The time & date to convert.
    @result A pointer to a NULL-terminated string containing a string representation of the given time & date.  Note: Unlike ctime, no line return is included. */

const char* timeAsString(struct timeval time);

/*! @function timeSinceNowAsString
    @abstract Returns a user-friendly string representation of the time between the current time and the given time.
    @discussion The returned string has the format up to "zz days, yy hours, xx minutes, vv seconds, uu milliseconds & tt microseconds [from now | ago]".  If for example the time is 5 seconds ago, the output will be "5 seconds, 0 milliseconds & 0 microseconds ago".
                Warning: the returned string points into an internal buffer.  Subsequent calls to this function will use the same buffer, overwriting existing values.  Make sure to copy or otherwise use this function after calling it, before using it again.
    @param time The time to compare to the curren time.
    @result A pointer to a NULL-terminated string containing a string representation of the given time & date. */

const char* timeSinceNowAsString(struct timeval time);

#endif
